/**
* DevExtreme (ui/load_panel.d.ts)
* Version: 21.1.3 (build 21154-1903)
* Build date: Thu Jun 03 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    UserDefinedElement,
    DxElement
} from '../core/element';

import {
    Cancelable,
    EventInfo,
    InitializedEventInfo,
    ChangedOptionInfo
} from '../events/index';

import {
    animationConfig
} from '../animation/fx';

import {
    positionConfig
} from '../animation/position';

import dxOverlay, {
    dxOverlayAnimation,
    dxOverlayOptions
} from './overlay';

export type ContentReadyEvent = EventInfo<dxLoadPanel>;

export type DisposingEvent = EventInfo<dxLoadPanel>;

export type HidingEvent = Cancelable & EventInfo<dxLoadPanel>;

export type HiddenEvent = EventInfo<dxLoadPanel>;

export type InitializedEvent = InitializedEventInfo<dxLoadPanel>;

export type OptionChangedEvent = EventInfo<dxLoadPanel> & ChangedOptionInfo;

export type ShowingEvent = EventInfo<dxLoadPanel>;

export type ShownEvent = EventInfo<dxLoadPanel>;

/**
 * @deprecated use Properties instead
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxLoadPanelOptions extends dxOverlayOptions<dxLoadPanel> {
    /**
     * Configures UI component visibility animations. This object contains two fields: show and hide.
     */
    animation?: dxLoadPanelAnimation;
    /**
     * Specifies the UI component&apos;s container.
     */
    container?: string | UserDefinedElement;
    /**
     * The delay in milliseconds after which the load panel is displayed.
     */
    delay?: number;
    /**
     * Specifies whether or not the UI component can be focused.
     */
    focusStateEnabled?: boolean;
    /**
     * 
     */
    height?: number | string | (() => number | string);
    /**
     * A URL pointing to an image to be used as a load indicator.
     */
    indicatorSrc?: string;
    /**
     * 
     */
    maxHeight?: number | string | (() => number | string);
    /**
     * 
     */
    maxWidth?: number | string | (() => number | string);
    /**
     * Specifies the text displayed in the load panel. Ignored in the Material Design theme.
     */
    message?: string;
    /**
     * 
     */
    position?: 'bottom' | 'center' | 'left' | 'left bottom' | 'left top' | 'right' | 'right bottom' | 'right top' | 'top' | positionConfig | Function;
    /**
     * 
     */
    shadingColor?: string;
    /**
     * A Boolean value specifying whether or not to show a load indicator.
     */
    showIndicator?: boolean;
    /**
     * A Boolean value specifying whether or not to show the pane behind the load indicator.
     */
    showPane?: boolean;
    /**
     * 
     */
    width?: number | string | (() => number | string);
}
/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxLoadPanelAnimation extends dxOverlayAnimation {
    /**
     * 
     */
    hide?: animationConfig;
    /**
     * 
     */
    show?: animationConfig;
}
/**
 * The LoadPanel is an overlay UI component notifying the viewer that loading is in progress.
 */
export default class dxLoadPanel extends dxOverlay {
    constructor(element: UserDefinedElement, options?: dxLoadPanelOptions)
}

export type Properties = dxLoadPanelOptions;

/**
 * @deprecated use Properties instead
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxLoadPanelOptions;

/**
 * @deprecated use Properties instead
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxLoadPanelOptions;
