/**
* DevExtreme (ui/popup.d.ts)
* Version: 21.1.3 (build 21154-1903)
* Build date: Thu Jun 03 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    animationConfig
} from '../animation/fx';

import {
    positionConfig
} from '../animation/position';

import {
    UserDefinedElement,
    DxElement
} from '../core/element';

import {
    template
} from '../core/templates/template';

import {
    Cancelable,
    EventInfo,
    NativeEventInfo,
    InitializedEventInfo,
    ChangedOptionInfo
} from '../events/index';

import dxOverlay, {
    dxOverlayAnimation,
    dxOverlayOptions
} from './overlay';

import {
    ResizeInfo
} from './resizable';

/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface TitleRenderedInfo {
    readonly titleElement: DxElement
}

export type ContentReadyEvent = EventInfo<dxPopup>;

export type DisposingEvent = EventInfo<dxPopup>;

export type HidingEvent = Cancelable & EventInfo<dxPopup>;

export type HiddenEvent = EventInfo<dxPopup>;

export type InitializedEvent = InitializedEventInfo<dxPopup>;

export type ShownEvent = EventInfo<dxPopup>;

export type ResizeEvent = NativeEventInfo<dxPopup> & ResizeInfo;

export type ResizeStartEvent = NativeEventInfo<dxPopup> & ResizeInfo;

export type ResizeEndEvent = NativeEventInfo<dxPopup> & ResizeInfo;

export type OptionChangedEvent = EventInfo<dxPopup> & ChangedOptionInfo;

export type ShowingEvent = EventInfo<dxPopup>;

export type TitleRenderedEvent = EventInfo<dxPopup> & TitleRenderedInfo;

/**
 * @deprecated use Properties instead
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPopupOptions<T = dxPopup> extends dxOverlayOptions<T> {
    /**
     * 
     */
    animation?: dxPopupAnimation;
    /**
     * Specifies the container in which to render the UI component.
     */
    container?: string | UserDefinedElement;
    /**
     * Specifies whether or not to allow a user to drag the popup window.
     */
    dragEnabled?: boolean;
    /**
     * 
     */
    focusStateEnabled?: boolean;
    /**
     * Specifies whether to display the Popup in full-screen mode.
     */
    fullScreen?: boolean;
    /**
     * 
     */
    height?: number | string | (() => number | string);
    /**
     * A function that is executed each time the UI component is resized by one pixel.
     */
    onResize?: ((e: ResizeEvent) => void);
    /**
     * A function that is executed when resizing ends.
     */
    onResizeEnd?: ((e: ResizeEndEvent) => void);
    /**
     * A function that is executed when resizing starts.
     */
    onResizeStart?: ((e: ResizeStartEvent) => void);
    /**
     * A function that is executed when the UI component&apos;s title is rendered.
     */
    onTitleRendered?: ((e: TitleRenderedEvent) => void);
    /**
     * 
     */
    position?: 'bottom' | 'center' | 'left' | 'left bottom' | 'left top' | 'right' | 'right bottom' | 'right top' | 'top' | positionConfig | Function;
    /**
     * Specifies whether or not an end user can resize the UI component.
     */
    resizeEnabled?: boolean;
    /**
     * Specifies whether or not the UI component displays the Close button.
     */
    showCloseButton?: boolean;
    /**
     * A Boolean value specifying whether or not to display the title in the popup window.
     */
    showTitle?: boolean;
    /**
     * The title in the overlay window.
     */
    title?: string;
    /**
     * Specifies a custom template for the UI component title. Does not apply if the title is defined.
     */
    titleTemplate?: template | ((titleElement: DxElement) => string | UserDefinedElement);
    /**
     * Configures toolbar items.
     */
    toolbarItems?: Array<dxPopupToolbarItem>;
    /**
     * 
     */
    width?: number | string | (() => number | string);
}
/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPopupAnimation extends dxOverlayAnimation {
    /**
     * 
     */
    hide?: animationConfig;
    /**
     * 
     */
    show?: animationConfig;
}
/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPopupToolbarItem {
    /**
     * Specifies whether the toolbar item responds to user interaction.
     */
    disabled?: boolean;
    /**
     * Specifies html code inserted into the toolbar item element.
     */
    html?: string;
    /**
     * Specifies a location for the item on the toolbar.
     */
    location?: 'after' | 'before' | 'center';
    /**
     * Configures the DevExtreme UI component used as a toolbar item.
     */
    options?: any;
    /**
     * Specifies a template that should be used to render this item only.
     */
    template?: template;
    /**
     * Specifies text displayed for the toolbar item.
     */
    text?: string;
    /**
     * Specifies whether the item is displayed on a top or bottom toolbar.
     */
    toolbar?: 'bottom' | 'top';
    /**
     * Specifies whether or not a UI component item must be displayed.
     */
    visible?: boolean;
    /**
     * A UI component that presents a toolbar item.
     */
    widget?: 'dxAutocomplete' | 'dxButton' | 'dxCheckBox' | 'dxDateBox' | 'dxMenu' | 'dxSelectBox' | 'dxTabs' | 'dxTextBox' | 'dxButtonGroup' | 'dxDropDownButton';
}
/**
 * The Popup UI component is a pop-up window overlaying the current view.
 */
export default class dxPopup extends dxOverlay {
    constructor(element: UserDefinedElement, options?: dxPopupOptions)
}

export type Properties = dxPopupOptions;

/**
 * @deprecated use Properties instead
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxPopupOptions;

/**
 * @deprecated use Properties instead
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxPopupOptions;
/**
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type ToolbarItem = dxPopupToolbarItem;
