/**
* DevExtreme (ui/range_slider.d.ts)
* Version: 21.1.3 (build 21154-1903)
* Build date: Thu Jun 03 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    UserDefinedElement
} from '../core/element';

import {
    EventInfo,
    NativeEventInfo,
    InitializedEventInfo,
    ChangedOptionInfo
} from '../events/index';

import {
    ValueChangedInfo
} from './editor/editor';

import {
    dxSliderBaseOptions
} from './slider';

import dxTrackBar from './track_bar';

export type ContentReadyEvent = EventInfo<dxRangeSlider>;

export type DisposingEvent = EventInfo<dxRangeSlider>;

export type InitializedEvent = InitializedEventInfo<dxRangeSlider>;

export type OptionChangedEvent = EventInfo<dxRangeSlider> & ChangedOptionInfo;

export type ValueChangedEvent = NativeEventInfo<dxRangeSlider> & ValueChangedInfo & {
    readonly start?: number;
    readonly end?: number;
    readonly value?: Array<number>;
}

/**
 * @deprecated use Properties instead
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxRangeSliderOptions extends dxSliderBaseOptions<dxRangeSlider> {
    /**
     * The right edge of the interval currently selected using the range slider.
     */
    end?: number;
    /**
     * The value to be assigned to the name attribute of the underlying `` element.
     */
    endName?: string;
    /**
     * 
     */
    onValueChanged?: ((e: ValueChangedEvent) => void);
    /**
     * The left edge of the interval currently selected using the range slider.
     */
    start?: number;
    /**
     * The value to be assigned to the name attribute of the underlying `` element.
     */
    startName?: string;
    /**
     * 
     */
    value?: Array<number>;
}
/**
 * The RangeSlider is a UI component that allows an end user to choose a range of numeric values.
 */
export default class dxRangeSlider extends dxTrackBar {
    constructor(element: UserDefinedElement, options?: dxRangeSliderOptions)
}

export type Properties = dxRangeSliderOptions;

/**
 * @deprecated use Properties instead
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxRangeSliderOptions;

/**
 * @deprecated use Properties instead
 * @deprecated Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxRangeSliderOptions;
